## ----global_options, include=TRUE, echo=FALSE---------------------------------
knitr::opts_chunk$set(
    echo = TRUE,
    warning = TRUE,
    message = TRUE,
    error = FALSE)

## ----'install', eval=FALSE, message=FALSE, warning=FALSE----------------------
#  # installing the BiocManager package if needed
#  if (!require("BiocManager", quietly = TRUE))
#      install.packages("BiocManager")
#  # installing the fgsea package from Bioconductor
#  BiocManager::install("fgsea")
#  # installing the mulea package from GitHub
#  devtools::install_github("https://github.com/ELTEbioinformatics/mulea")

## ----'DE1', eval=FALSE, message=FALSE, warning=FALSE--------------------------
#  library(tidyverse)
#  geo2r_result_tab <- read_tsv("GSE55662.table_wt_non_vs_cipro.tsv")

## ----'DE1_eval', eval=TRUE, message=FALSE, warning=FALSE, echo=FALSE----------
library(tidyverse)
geo2r_result_tab <- 
    read_tsv("../inst/extdata/GSE55662.table_wt_non_vs_cipro.tsv")

## ----'print_geo', echo=FALSE--------------------------------------------------
geo2r_result_tab %>%  
    head(3) %>% 
    knitr::kable()

## ----'format_geo'-------------------------------------------------------------
geo2r_result_tab %<>% 
    # extracting the first gene symbol from the Gene.symbol column
    mutate(Gene.symbol = str_remove(
        string = Gene.symbol,
        pattern = "\\/.*")) %>% 
    # removing rows where Gene.symbol is NA
    filter(!is.na(Gene.symbol)) %>% 
    # ordering by logFC
    arrange(desc(logFC))

## ----'print_geo_formatted', echo=FALSE----------------------------------------
geo2r_result_tab %>%  
    head(3) %>% 
    knitr::kable()

## ----'read_GMT1', eval=FALSE--------------------------------------------------
#  library(mulea)
#  tf_gmt <-
#      read_gmt("Transcription_factor_RegulonDB_Escherichia_coli_GeneSymbol.gmt")

## ----'read_GMT2', echo=FALSE--------------------------------------------------
library(mulea)
tf_gmt <- read_gmt("../inst/extdata/Transcription_factor.gmt")

## ----'print_nr_of_tf'---------------------------------------------------------
nrow(tf_gmt)

## ----'print_reulon', echo=FALSE-----------------------------------------------
tf_gmt %>%
    head(3) %>% 
    knitr::kable()

## ----'print_acrr'-------------------------------------------------------------
tf_gmt %>% 
    # filtering the row where the ontology_id is AcrR
    filter(ontology_id == "AcrR") %>% 
    # selecting the list_of_values column
    select(list_of_values) %>% 
    # converting tibble to vector
    pull()

## ----'plot_list_of_values'----------------------------------------------------
Nr_of_elements_in_ontology <- tf_gmt$list_of_values %>% 
    map_dbl(length)
ggplot(mapping = aes(Nr_of_elements_in_ontology)) + 
    geom_bar() +
    theme_minimal()

## ----'plot_list_of_values_zoom', warning=FALSE--------------------------------
ggplot(mapping = aes(Nr_of_elements_in_ontology)) + 
    geom_bar() +
    xlim(0, 15) +
    theme_minimal()

## ----'exclude_ontology'-------------------------------------------------------
tf_gmt_filtered <- filter_ontology(
    gmt = tf_gmt,
    min_nr_of_elements = 3,
    max_nr_of_elements = 400)

## ----'print_nr_of_tf_filtered'------------------------------------------------
nrow(tf_gmt_filtered)

## ----'save_gmt', eval=FALSE---------------------------------------------------
#  write_gmt(gmt = tf_gmt_filtered, file = "Filtered.gmt")

## ----'list_to_gmt_example', eval=FALSE----------------------------------------
#  # creating a list of gene sets
#  ontology_list <- list(
#      gene_set1 = c("gene1", "gene2", "gene3"),
#      gene_set2 = c("gene4", "gene5", "gene6"))
#  
#  # converting the list to a ontology (GMT) object
#  new_ontology_object <- list_to_gmt(ontology_list)

## ----'target_set'-------------------------------------------------------------
sign_genes <- geo2r_result_tab %>% 
    # filtering for adjusted p-value < 0.05 and logFC > 1
    filter(adj.P.Val < 0.05 & logFC > 1) %>% 
    # selecting the Gene.symbol column
    select(Gene.symbol) %>% 
    # converting the tibble to a vector
    pull() %>% 
    # removing duplicates
    unique()

## ----'target_head'------------------------------------------------------------
sign_genes %>% 
    head(10)

## ----'target_gene_nr'---------------------------------------------------------
sign_genes %>% 
    length()

## ----'background_set'---------------------------------------------------------
background_genes <- geo2r_result_tab %>% 
    # selecting the Gene.symbol column
    select(Gene.symbol) %>% 
    # convertin the tibble to a vector
    pull() %>% 
    # removing duplicates
    unique()

## ----'background_gene_nr'-----------------------------------------------------
background_genes %>% 
    length()

## ----'ora'--------------------------------------------------------------------
# creating the ORA model using the GMT variable
ora_model <- ora(
    gmt = tf_gmt_filtered, 
    # the test set variable
    element_names = sign_genes,
    # the background set variable
    background_element_names = background_genes,
    # the p-value adjustment method
    p_value_adjustment_method = "eFDR",
    number_of_permutations = 10000,
    # the number of processor threads to use
    nthreads = 1) 
# running the ORA
ora_results <- run_test(ora_model)

## ----'ora_size'---------------------------------------------------------------
ora_results %>%
    # rows where the eFDR < 0.05
    filter(eFDR < 0.05) %>% 
    # the number of such rows
    nrow()

## ----'print_ora', eval=FALSE--------------------------------------------------
#  ora_results %>%
#      # arrange the rows by the eFDR values
#      arrange(eFDR) %>%
#      # rows where the eFDR < 0.05
#      filter(eFDR < 0.05)

## ----'print_ora2', echo=FALSE-------------------------------------------------
ora_results %>%
    # arrange the rows by the eFDR values
    arrange(eFDR) %>% 
    # rows where the eFDR < 0.05
    filter(eFDR < 0.05) %>% 
    knitr::kable()

## ----'init_plot_ora'----------------------------------------------------------
ora_reshaped_results <- reshape_results(
    model = ora_model,
    model_results = ora_results,
    # choosing which column to use for the indication of significance
    p_value_type_colname = "eFDR")

## ----'lollipop_plot_ora'------------------------------------------------------
plot_lollipop(
    reshaped_results = ora_reshaped_results,
    # the column containing the names we wish to plot
    ontology_id_colname = "ontology_id",
    # upper threshold for the value indicating the significance
    p_value_max_threshold = 0.05,
    # column that indicates the significance values
    p_value_type_colname = "eFDR")

## ----'bar_plot_ora'-----------------------------------------------------------
plot_barplot(
    reshaped_results = ora_reshaped_results,
    # the column containing the names we wish to plot
    ontology_id_colname = "ontology_id",
    # upper threshold for the value indicating the significance
    p_value_max_threshold = 0.05,
    # column that indicates the significance values
    p_value_type_colname = "eFDR")

## ----'network_plot_ora'-------------------------------------------------------
plot_graph(
    reshaped_results = ora_reshaped_results,
    # the column containing the names we wish to plot
    ontology_id_colname = "ontology_id",
    # upper threshold for the value indicating the significance
    p_value_max_threshold = 0.05,
    # column that indicates the significance values
    p_value_type_colname = "eFDR")

## ----'heatmap_ora'------------------------------------------------------------
plot_heatmap(
    reshaped_results = ora_reshaped_results,
    # the column containing the names we wish to plot
    ontology_id_colname = "ontology_id",
    # column that indicates the significance values
    p_value_type_colname = "eFDR")

## ----'gsea_input'-------------------------------------------------------------
# if there are duplicated Gene.symbols keep the first one only
geo2r_result_tab_filtered <- geo2r_result_tab %>% 
    # grouping by Gene.symbol to be able to filter
    group_by(Gene.symbol) %>%
    # keeping the first row for each Gene.symbol from rows with the same 
    # Gene.symbol
    filter(row_number()==1) %>% 
    # ungrouping
    ungroup() %>% 
    # arranging by logFC in descending order
    arrange(desc(logFC)) %>%
    select(Gene.symbol, logFC)

## ----'ordered_genes_length'---------------------------------------------------
geo2r_result_tab_filtered %>% 
    nrow()

## ----'gsea', warning=FALSE, message=FALSE-------------------------------------
# creating the GSEA model using the GMT variable
gsea_model <- gsea(
    gmt = tf_gmt_filtered,
    # the names of elements to test
    element_names = geo2r_result_tab_filtered$Gene.symbol,
    # the logFC-s of elements to test
    element_scores = geo2r_result_tab_filtered$logFC,
    # consider elements having positive logFC values only
    element_score_type = "pos",
    # the number of permutations
    number_of_permutations = 10000)
# running the GSEA
gsea_results <- run_test(gsea_model)

## ----'gsea_size'--------------------------------------------------------------
gsea_results %>%
    # rows where the adjusted_p_value < 0.05
    filter(adjusted_p_value < 0.05) %>% 
    # the number of such rows
    nrow()

## ----'print_gsea', eval=FALSE-------------------------------------------------
#  gsea_results %>%
#      # arrange the rows by the adjusted_p_value values
#      arrange(adjusted_p_value) %>%
#      # rows where the adjusted_p_value < 0.05
#      filter(adjusted_p_value < 0.05)

## ----'print_gsea2', echo=FALSE------------------------------------------------
gsea_results %>%
    # arrange the rows by the adjusted_p_value values
    arrange(adjusted_p_value) %>% 
    # rows where the adjusted_p_value < 0.05
    filter(adjusted_p_value < 0.05) %>% 
    knitr::kable()

## ----'init_plot_gsea'---------------------------------------------------------
gsea_reshaped_results <- reshape_results(
    model = gsea_model,
    model_results = gsea_results,
    # choosing which column to use for the indication of significance
    p_value_type_colname = "adjusted_p_value")

## ----'lollipop_plot_gsea'-----------------------------------------------------
plot_lollipop(
    reshaped_results = gsea_reshaped_results,
    # the column containing the names we wish to plot
    ontology_id_colname = "ontology_id",
    # upper threshold for the value indicating the significance
    p_value_max_threshold = 0.05,
    # column that indicates the significance values
    p_value_type_colname = "adjusted_p_value")

## ----'bar_plot_gsea'----------------------------------------------------------
plot_barplot(
    reshaped_results = gsea_reshaped_results,
    # the column containing the names we wish to plot
    ontology_id_colname = "ontology_id",
    # upper threshold for the value indicating the significance
    p_value_max_threshold = 0.05,
    # column that indicates the significance values
    p_value_type_colname = "adjusted_p_value")

## ----'network_plot_gsea'------------------------------------------------------
plot_graph(
    reshaped_results = gsea_reshaped_results,
    # the column containing the names we wish to plot
    ontology_id_colname = "ontology_id",
    # upper threshold for the value indicating the significance
    p_value_max_threshold = 0.05,
    # column that indicates the significance values
    p_value_type_colname = "adjusted_p_value")

## ----'heatmap_gsea', fig.width=20---------------------------------------------
plot_heatmap(
    reshaped_results = gsea_reshaped_results,
    # the column containing the names we wish to plot
    ontology_id_colname = "ontology_id",
    # column that indicates the significance values
    p_value_type_colname = "adjusted_p_value")

## ----'session_info'-----------------------------------------------------------
sessionInfo()

